﻿----------------------------------------------------------------------------
----Vehicle Roll-Over Script Version 1.0.1.0--------------------------------
----------------------------------------------------------------------------
----Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is always the best design." -------
----------------------------------------------------------------------------

rollOver = {};

function rollOver.prerequisitesPresent(specializations)

    return SpecializationUtil.hasSpecialization(Motorized, specializations); --Must have "Motorized" specialization to use script.

end;

function rollOver:load(savegame)
	self.rollForce = 0.66;
	self.renderHelp = false;--Boolean used for toggling the Help Menu controls.	
	vL = 1.1; --Velocity Limit.
	self.xmlSize = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.fieldJob#sizeCategory"), 0); --Attemps reading from vehicle xml to get the vehicle size.

	if self.xmlSize == 1 then
		self.rollForce = 0.22;
	elseif self.xmlSize == 2 then
		self.rollForce = 0.44; 
	end;	--anything bigger than size 2 gets default of 0.66

	self.noRollOver = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.rollOver#disabled"), false); --Attemps reading from vehicle xml to disable Roll-Over if detected.
	self.rollForce = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rollOver#impulse"), self.rollForce); --Attemps reading from vehicle xml to change the impulse amount if deteced.

	if self.rollForce > 1.1 then --Limit the (Max) amount of adjusting allowed
		 self.rollForce = 1.1;	
	elseif  self.rollForce < 0.05 then --Limit the (Min) amount of adjusting allowed
		 self.rollForce = 0.05;
	end;

end;

function rollOver:delete()
end;

function rollOver:mouseEvent(posX, posY, isDown, isUp, button)
end;

function rollOver:keyEvent(unicode, sym, modifier, isDown)
end;

function rollOver:updateTick(dt)
end;

function rollOver:update(dt)

	if g_currentMission.controlledVehicle == self then  -- If self is a controllable vehicle then..

		if (self.isBroken == false) and (self.noRollOver == false) then --If vehicle is not broken and "Roll-Over" is not disabled in the XML then..

			if InputBinding.hasEvent(InputBinding.rollOverKey1) then -- Roll

				local node = self.components[1].node;
				local rx,ry,rz = getRotation(node); --Rotation values are in pi format. 90° = pi/2 , 180°\-180° = pi\-pi , -90° = -pi/2
				rx, ry, rz = worldToLocal(node, rx, ry, rz); -- Changes pi format to standardized.
				local cx,cy,cz = getCenterOfMass(node); -- Used to set the load's c.o.m. prior to impulse.
				local vx,vy,vz = getAngularVelocity(node); --Get Rotational Velocity for a desired threshold.
				local power = self:getTotalMass(true,false); --Get mass of vehicle plus all attachments to even the amount of force per vehicle.

				power  = power  * self.rollForce; --Amount of force.

				setCenterOfMass(node, 0.0, 0.5, 0.0); -- Adjust center of mass prior to flip

				if (vx < vL) and (vx > -vL) and (vy < vL) and (vy > -vL) and (vz < vL) and (vz > -vL) then --Prevent velocity build up from extremes.
					if rz >= 0.0 and rz <= 244.0 then
						addImpulse(self.rootNode, 0.0, power, 0.0, 15.0, 10.0, 0.0, true); 	--(targetID, Ximpulse, Yimpulse, Zimpulse, XLocation, YLocation, ZLocation, Local=True) (location works as an offset (to cause a roll))			
					else			
						addImpulse(self.rootNode, 0.0, power, 0.0, -15.0, 10.0, 0.0, true);
					end;
				end;

				setCenterOfMass(node, cx,cy,cz);  -- Reset center of mass		
			end;

			if InputBinding.hasEvent(InputBinding.rollOverKey2) then -- Reverse Roll	
			
				local node = self.components[1].node;
				local rx,ry,rz = getRotation(node);
				rx, ry, rz = worldToLocal(node, rx, ry, rz);
				local cx,cy,cz = getCenterOfMass(node);
				local vx,vy,vz = getAngularVelocity(node);
				local power = self:getTotalMass(true,false);

				power  = power  * self.rollForce;
			
				setCenterOfMass(node, 0.0, 0.5, 0.0);

				if (vx < vL) and (vx > -vL) and (vy < vL) and (vy > -vL) and (vz < vL) and (vz > -vL) then
					if rz >= 0.0 and rz <= 244.0 then
						addImpulse(self.rootNode, 0.0, power, 0.0, -15.0, 10.0, 0.0, true);	-- Reversed
					else			
						addImpulse(self.rootNode, 0.0, power, 0.0, 15.0, 10.0, 0.0, true);-- Reversed
					end;
				end;
			
				setCenterOfMass(node, cx,cy,cz);

			end;

			if InputBinding.hasEvent(InputBinding.rollOverKey3) then --(Stop rolling) (requires repeated presses)
			
				setAngularVelocity(self.rootNode, 0.0, 0.0, 0.0); -- Zero Angular Velocity
				setLinearVelocity(self.rootNode, 0.0, 0.0, 0.0); -- Zero Linear Velocity			

			end;

			if InputBinding.hasEvent(InputBinding.rollOverKeyH) then --(toggles the control list in the help menu)

				if self.renderHelp then
					self.renderHelp = false;
				else
					self.renderHelp = true;
				end;
			end;
		end;	
	end;

end;

function rollOver:draw()
	if self.noRollOver == false then
		if self.renderHelp then
			g_currentMission:addHelpButtonText(g_i18n:getText("input_rollOverKeyH2"), InputBinding.rollOverKeyH, nil, GS_PRIO_HIGH);
			g_currentMission:addHelpButtonText(g_i18n:getText("input_rollOverKey1"), InputBinding.rollOverKey1, nil, GS_PRIO_HIGH);
			g_currentMission:addHelpButtonText(g_i18n:getText("input_rollOverKey2"), InputBinding.rollOverKey2, nil, GS_PRIO_HIGH);
			g_currentMission:addHelpButtonText(g_i18n:getText("input_rollOverKey3"), InputBinding.rollOverKey3, nil, GS_PRIO_HIGH);
		else	
			g_currentMission:addHelpButtonText(g_i18n:getText("input_rollOverKeyH1"), InputBinding.rollOverKeyH, nil, GS_PRIO_HIGH);
		end;	
	end;
end;

function rollOver:onLeave()
end;